'''
Dialog box used to create STAMP profiles

@author: Donovan Parks
'''

import string

from PyQt4 import QtGui, QtCore
from createProfileUI import Ui_CreateProfileDlg
from GUI.customizeHeadingsDlg import CustomizeHeadingsDlg

class ProfileRow():
  def __init__(self):
    countData = []
    hierarchy = []

class CreateProfileDlg(QtGui.QDialog):
  def __init__(self, parent=None):
    QtGui.QWidget.__init__(self, parent)
    
    # initialize GUI
    self.ui = Ui_CreateProfileDlg()
    self.ui.setupUi(self)

    self.centerWindow()
    
    QtCore.QObject.connect(self.ui.cboProfileType, QtCore.SIGNAL('activated(QString)'), self.changeProfileType)
    QtCore.QObject.connect(self.ui.btnLoadProfiles, QtCore.SIGNAL("clicked()"), self.loadProfiles)
    QtCore.QObject.connect(self.ui.btnCustomizeHeadings, QtCore.SIGNAL("clicked()"), self.customizeHeadings)
    QtCore.QObject.connect(self.ui.btnCreateProfile, QtCore.SIGNAL("clicked()"), self.createProfile)
    QtCore.QObject.connect(self.ui.btnCancel, QtCore.SIGNAL("clicked()"), self.accept)
    
    self.headings = []
    self.changeProfileType()
    
    self.selectedFiles = []
       
  def changeProfileType(self):
    self.headings = []
    if self.ui.cboProfileType.currentText() == 'MG-RAST metabolic profile':    
      self.headings.append('Subsystem Hierarchy 1')
      self.headings.append('Subsystem Hierarchy 2')
      self.headings.append('Subsystem Name')
      self.headings.append('')
      self.headings.append('')
      self.headings.append('')
    elif self.ui.cboProfileType.currentText() == 'MG-RAST phylogenetic profile':  
      self.headings.append('Domain')
      self.headings.append('Level 2')
      self.headings.append('Level 3')
      self.headings.append('Level 4')
      self.headings.append('Level 5')
      self.headings.append('Organism Name')
    
  def loadProfiles(self):
    selectedFiles = QtGui.QFileDialog.getOpenFileNames(self, 'Load profiles', '', 'MG-RAST profiles (*.tsv);;All files (*.*)')

    if len(selectedFiles) > 0:
      for file in selectedFiles:
        self.selectedFiles.append(str(file))
        self.ui.lstSelectedProfiles.addItem(file)
        self.ui.btnCustomizeHeadings.setEnabled(True)
        self.ui.btnCreateProfile.setEnabled(True)
        
    
  def customizeHeadings(self):
    customizeHeadingsDlg = CustomizeHeadingsDlg(self) 
    
    if self.ui.cboProfileType.currentText() == 'MG-RAST metabolic profile':    
      customizeHeadingsDlg.ui.txtInfo.setText('MG-RAST metabolic profiles consist of three hierarchical levels.')        
    elif self.ui.cboProfileType.currentText() == 'MG-RAST phylogenetic profile':  
      customizeHeadingsDlg.ui.txtInfo.setText('MG-RAST phylogenetic profiles consist of five hierarchical levels.')
   
    customizeHeadingsDlg.ui.txtLevel1.setText(self.headings[0])
    customizeHeadingsDlg.ui.txtLevel2.setText(self.headings[1])
    customizeHeadingsDlg.ui.txtLevel3.setText(self.headings[2])
    customizeHeadingsDlg.ui.txtLevel4.setText(self.headings[3])
    customizeHeadingsDlg.ui.txtLevel5.setText(self.headings[4])
    customizeHeadingsDlg.ui.txtLevel6.setText(self.headings[5])
           
    if customizeHeadingsDlg.exec_() == QtGui.QDialog.Accepted:
      self.headings[0] = customizeHeadingsDlg.ui.txtLevel1.text()
      self.headings[1] = customizeHeadingsDlg.ui.txtLevel2.text()
      self.headings[2] = customizeHeadingsDlg.ui.txtLevel3.text()
      self.headings[3] = customizeHeadingsDlg.ui.txtLevel4.text()
      self.headings[4] = customizeHeadingsDlg.ui.txtLevel5.text()
      self.headings[5] = customizeHeadingsDlg.ui.txtLevel6.text()
  
  def createProfile(self):
    # get filename to save STAMP profile to
    stampFilename = QtGui.QFileDialog.getSaveFileName(self, 'Save STAMP profile...', '',
                          'STAMP profile file(*.spf);;All files(*.*)')
    if stampFilename == '':
      return
    

    # set profile specific parsing information
    if self.ui.cboProfileType.currentText() == 'MG-RAST metabolic profile':    
      splitCh = '\t'
      dataIndex = 3        
    elif self.ui.cboProfileType.currentText() == 'MG-RAST phylogenetic profile':  
      splitCh = '\t'
      dataIndex = 6
       

    # get profile information from each file
    profileDict = {} 
    profileIndex = 0
    sampleNames = []
    for file in self.selectedFiles:
      fin = open(file, 'U')
      data = map(string.strip, fin.readlines())
      fin.close()
      
      sampleNames.append(file[file.rfind('/')+1:file.rfind('.')])
      
      # add profile info
      for i in xrange(1, len(data)):
        if data[i] == "":
            continue  # skip blank lines
        
        lineSplit = data[i].split(splitCh)
        if len(lineSplit) != dataIndex+1:
            QtGui.QMessageBox.information(self, 'Unrecognized file format', 'You file does not appear to be a ' + self.ui.cboProfileType.currentText() + '.')
            return
        
        count = int(lineSplit[dataIndex])
        hierarchy = lineSplit[0:dataIndex]
        
        row = profileDict.get(hierarchy[-1], None)
        if row == None:
          row = ProfileRow()
          row.countData = [0] * len(self.selectedFiles)
          row.hierarchy = hierarchy
          profileDict[hierarchy[-1]] = row
          
        row.countData[profileIndex] += count
        
      profileIndex += 1
      
    # write out STAMP profile
    try:
      fout = open(stampFilename, 'w')
    except IOError:
      QtGui.QMessageBox.information(self, 'Failed to save STAMP profile', 'Write permission for file denied.', QtGui.QMessageBox.Ok)
      return

    fout.write(self.headings[0])
    for heading in self.headings[1:dataIndex]:
      fout.write('\t' + heading)
      
    for sampleName in sampleNames:
      fout.write('\t' + sampleName)
    fout.write('\n')
      
    for key in profileDict.keys():
      row = profileDict[key]
      for h in row.hierarchy:
        fout.write(h + '\t')
      
      fout.write(str(row.countData[0]))
      for c in row.countData[1:]:
        fout.write('\t' + str(c))
      fout.write('\n')
      
    fout.close()
        
    self.accept()

  def centerWindow(self):
    screen = QtGui.QDesktopWidget().screenGeometry()
    size =  self.geometry()
    self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)
